package com.xsnb.system.controller;


import com.xsnb.common.result.Result;
import com.xsnb.system.config.MinioConfig;
import com.xsnb.system.utils.minio.MinioUtil;
import io.minio.messages.Bucket;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;


@Slf4j
@RestController
@RequestMapping(value = "product/file")
public class FileController {
    @Autowired
    private MinioUtil minioUtil;


    @Autowired
    private MinioConfig prop;

    //@ApiOperation(value = "查看存储bucket是否存在")
    @GetMapping("/bucketExists")
    public Result bucketExists(@RequestParam("bucketName") String bucketName) {
        return Result.success(minioUtil.bucketExists(bucketName));
    }

    //@ApiOperation(value = "创建存储bucket")
    @GetMapping("/makeBucket")
    public Result makeBucket(String bucketName) {
        return Result.success(minioUtil.makeBucket(bucketName));
    }

    //@ApiOperation(value = "删除存储bucket")
    @GetMapping("/removeBucket")
    public Result removeBucket(String bucketName) {
        return Result.success(minioUtil.removeBucket(bucketName));
    }

    //@ApiOperation(value = "获取全部bucket")
    @GetMapping("/getAllBuckets")
    public Result getAllBuckets() {
        List<Bucket> allBuckets = minioUtil.getAllBuckets();
        return Result.success(allBuckets);
    }

    //@ApiOperation(value = "文件上传返回url")
    @CrossOrigin(origins = "http://192.168.2.143:8080") // 允许来自http://localhost:8080的跨域请求
    @PostMapping("/upload")
    public Result upload(@RequestParam("file") MultipartFile file) {
        String objectName = minioUtil.upload(file);
        if (null != objectName) {
            return Result.success((prop.getEndpoint() + "/" + prop.getBucketName() + "/" + objectName), "url");
        }
        return Result.success("请求失败");
    }


    //@ApiOperation(value = "图片/视频预览")
    @GetMapping("/preview")
    public Result preview(@RequestParam("fileName") String fileName) {
        String preview = minioUtil.preview(fileName);
        return Result.success(preview);
    }

    //@ApiOperation(value = "文件下载")
    @GetMapping("/download")
    public Result download(@RequestParam("fileName") String fileName, HttpServletResponse res) {
        minioUtil.download(fileName, res);
        return Result.success("下载成功");
    }

    //@ApiOperation(value = "删除文件", notes = "根据url地址删除文件")
    @PostMapping("/delete")
    public Result remove(String url) {
        String objName = url.substring(url.lastIndexOf(prop.getBucketName() + "/") + prop.getBucketName().length() + 1);
        minioUtil.remove(objName);
        return Result.success("删除成功" + objName);
    }

}

