package com.xsnb.system.controller;

import com.xsnb.common.domain.response.RespJwt;
import com.xsnb.common.result.Result;
import com.xsnb.system.service.WechatService;
import com.xsnb.system.utils.GzhUtiles;
import com.xsnb.system.utils.HttpUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@RestController
@RequestMapping("/weChat")
public class WechatController {


    @Autowired
    GzhUtiles gzhUtiles;

    @Resource
    private WechatService wechatService;


    private HttpUtil httpUtil;

    @Autowired
    public WechatController(HttpUtil httpUtil) {
        this.httpUtil = httpUtil;
    }


    /**
     * 获取登入二维码
     *
     * @return
     */
    @GetMapping("/getLoginQrCode")
    @ResponseBody
    public Result getLoginQrCode() {
        return Result.success(wechatService.getQRCode(), "获取成功");
    }

    /**
     * 轮询查看微信的登录状态，是否可以登录，简单示例，想复杂和安全点可以使用ws
     *
     * @param ticket 是微信扫码返回的uuid
     * @return
     */
    @GetMapping("/checkLogin")
    public Result<RespJwt> checkLogin(String ticket) {
        RespJwt respJwt = wechatService.checkLogin(ticket);

        return Result.success(respJwt);
    }

    //微信返回不同请求-post请求
    @PostMapping(value = "/weCat.do", produces = {"application/json;charset=UTF-8"})
    public void validationWeCat(@RequestBody String requestBody,
                                HttpServletResponse response)
            throws IOException {
        wechatService.validationWeCat(requestBody);
        response.getOutputStream().println("success");
    }

    //微信返回不同请求-get请求
    @GetMapping(value = "/weCat.do", produces = {"application/json;charset=UTF-8"})
    public void validationWeCat(@RequestParam String signature,
                                @RequestParam String timestamp,
                                @RequestParam String nonce,
                                @RequestParam String echostr,
                                HttpServletResponse response) throws IOException {

        final boolean b = gzhUtiles.checkQianMing(signature, timestamp, nonce);
        if (b) {
            response.getOutputStream().println(echostr);
        } else {
            System.out.println("token验证失败");
        }
    }
}
