package com.xsnb.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xsnb.common.domain.User;
import com.xsnb.common.result.Result;
import com.xsnb.system.domain.Attachment;
import com.xsnb.system.domain.Mail;
import com.xsnb.system.mapper.AttachmentMapper;
import com.xsnb.system.mapper.MailMapper;
import com.xsnb.system.service.MailService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * FileName: MailServiceImpl
 * Author: 张海宁
 * Date:   2024/3/4 14:18
 * Description:
 * History:
 * <author>      <time>     <version>     <desc>
 * 作者姓名       修改时间      版本号         描述
 */
@Service
@Log4j2
public class MailServiceImpl implements MailService {
    @Autowired
    MailMapper mapper;
    @Autowired
    UserServiceImpl userService;

    @Autowired
    AttachmentMapper attachmentMapper;

    /**
     * @param user
     * @return
     */
    @Override
    public Result<List<Mail>> allListMail(User user) {
        QueryWrapper<Mail> qw = new QueryWrapper<>();
        Integer id = user.getId();
        log.info("id的值是:{}", id);
        qw.eq("recipient_id", id);
        log.info("qw值是:{}", qw);
        //通过接收者id来查询邮件
        List<Mail> mail = mapper.selectList(qw);
        return Result.success(mail);
    }

    /**
     * @param delIds
     * @return
     */
    @Override
    public Result deleteMail(Long[] delIds) {
        System.out.println("获取的delIds是:"+delIds.toString());
        return Result.success("批量删除成功,未删除的邮件为");
    }

    /**
     * @param mail
     * @return
     */
    @Override
    public Result<Mail> addMail(Mail mail) {
        log.info("添加邮件的数据:{}", mail);
        //获取当前用户信息
        Mail.MailBuilder builder = Mail
                .builder()
                .title(mail.getTitle())
                .sendTime(new Date())
                .readTime(mail.getReadTime())
                .content(mail.getContent())
                .attachmentId(mail.getAttachmentId())
                .emailStatus("0")
                .userId(mail.getUserId())
                .recipientId(mail.getRecipientId());
        //存入数据库
        int insert = mapper.insert(builder.build());
        log.info("添加邮件的结果:{}", builder);
        return insert == 1 ? Result.success(mail, "发送成功") : Result.error("发送失败");
    }

    /**
     * 修改为已读
     *
     * @param mail
     * @return
     */
    @Override
    public Result reading(Mail mail) {
        Mail mail1 = mail;
        mail1.setEmailStatus("1");
        int i = mapper.updateById(mail1);
        return i == 1 ? Result.success(mail, "修改成功") : Result.error("修改失败");
    }

    /**
     * @param mail
     * @return
     */
    @Override
    public Result addAttachment(Mail mail, MultipartFile file) {
        // 如果邮件对象或文件对象为空，返回错误结果
        if (mail == null || file == null || file.isEmpty()) {
            return Result.error("邮件对象或文件为空");
        }
        try {
            // 生成唯一的文件名
            String fileName = UUID.randomUUID().toString() + "_" + file.getOriginalFilename();
            // 构建文件保存路径
            Path filePath = Paths.get("D://file" + File.separator + fileName);
            // 将文件保存到本地文件系统
            Files.copy(file.getInputStream(), filePath);
            int id = mail.getId();
            Attachment build = new Attachment().builder().emailId(String.valueOf(id)).fileName(fileName).filePath(filePath.toString()).upUserId(String.valueOf(mail.getUserId())).build();
            int insert = attachmentMapper.insert(build);
            // 返回成功结果
            return Result.success("文件上传成功", filePath.toString());
        } catch (IOException e) {
            e.printStackTrace();
            // 文件上传失败，返回错误结果
            return Result.error("文件上传失败");
        }
    }

    /**
     * @param mail
     * @return
     */
    @Override
    public Result copyMail(Mail mail) {



        return null;
    }


}
