package com.xsnb.system.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xsnb.common.domain.XmlData;
import org.apache.commons.lang3.StringEscapeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component
public class GzhUtiles {


    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    private final Environment env;

    private static String AppSecret;
    private static String appid;
    private static String TOKEN;

    @Autowired
    public GzhUtiles(Environment env) {
        this.env = env;

        AppSecret = env.getProperty("wx.AppSecret");
        appid = env.getProperty("wx.appid");
        TOKEN = env.getProperty("wx.gzhToken");

    }


    /**
     * 公众号验证token--验证签名
     *
     * @param signature
     * @param timestamp
     * @param nonce
     * @return
     */
    public boolean checkQianMing(String signature, String timestamp, String nonce) {
        String[] tmpArr = {TOKEN, timestamp, nonce};
        Arrays.sort(tmpArr);
        StringBuilder tmpStrBuilder = new StringBuilder();
        for (String str : tmpArr) {
            tmpStrBuilder.append(str);
        }
        String tmpStr = tmpStrBuilder.toString();
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            byte[] digest = md.digest(tmpStr.getBytes());
            StringBuilder hexStrBuilder = new StringBuilder();
            for (byte b : digest) {
                hexStrBuilder.append(String.format("%02x", b));
            }
            String calculatedSignature = hexStrBuilder.toString();
            return calculatedSignature.equals(signature);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            return false;
        }
    }


    /**
     * 获取微信公众号的AccessToken
     *
     * @return
     */
    public String getToken() {
        // 判断数据是否存在
        Boolean gzhAccessToken = redisTemplate.hasKey("gzh_access_token");
        //存入缓存  存在两小时
        //三秒
        if (!gzhAccessToken) {
            // 授予形式
            System.out.println("第一次进入");
            String grant_type = "client_credential";
            // 接口地址拼接参数(appid为微信服务号的appid，secret为服务号的秘钥)
            String getTokenApi = "https://api.weixin.qq.com/cgi-bin/token?grant_type=" + grant_type + "&appid=" + appid
                    + "&secret=" + AppSecret;
            String tokenJsonStr = doGetPost(getTokenApi, "GET", null);
            JSONObject tokenJson = JSONObject.parseObject(tokenJsonStr);
            String token = tokenJson.get("access_token").toString();
            // 存入缓存 120分钟 重新获取
            // 存储数据并设置过期时间为 120 分钟
            redisTemplate.opsForValue().set("gzh_access_token", token, 2, TimeUnit.HOURS);
            return token;
        } else {
            // 获取数据
            return redisTemplate.opsForValue().get("gzh_access_token");
        }
    }


    /**
     * 调用接口 post
     *
     * @param apiPath
     */
    public static String doGetPost(String apiPath, String type, Map<String, Object> paramMap) {
        OutputStreamWriter out = null;
        InputStream is = null;
        String result = null;

        try {
            URL url = new URL(apiPath);// 创建连接
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setUseCaches(false);
            connection.setInstanceFollowRedirects(true);
            connection.setRequestMethod(type); // 设置请求方式
            connection.setRequestProperty("Accept", "application/json"); // 设置接收数据的格式
            connection.setRequestProperty("Content-Type", "application/json"); // 设置发送数据的格式
            connection.connect();
            if (type.equals("POST")) {
                out = new OutputStreamWriter(connection.getOutputStream(), "UTF-8"); // utf-8编码
                out.append(JSON.toJSONString(paramMap));
                out.flush();
                out.close();
            }
            // 读取响应
            is = connection.getInputStream();
            int length = (int) connection.getContentLength();// 获取长度
            if (length != -1) {
                byte[] data = new byte[length];
                byte[] temp = new byte[512];
                int readLen = 0;
                int destPos = 0;
                while ((readLen = is.read(temp)) > 0) {
                    System.arraycopy(temp, 0, data, destPos, readLen);
                    destPos += readLen;
                }
                result = new String(data, "UTF-8"); // utf-8编码
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }


    /**
     * 解析公众号用户触发响应的xml
     *
     * @param requestBody
     * @return
     */
    public static XmlData jxXml(String requestBody) {
        String unescapedXml = StringEscapeUtils.unescapeHtml4(requestBody);
        try {
            return (XmlData)JAXBContext
                    .newInstance(XmlData.class)
                    .createUnmarshaller()
                    .unmarshal(
                            new StringReader(unescapedXml)
                    );
        } catch (JAXBException e) {
            e.printStackTrace();
        }
        return null;
    }

}
